/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Areski Flissi, Christophe Demarey.

====================================================================*/

package org.objectweb.openccm.corba;

// Package dependencies
import java.net.InetAddress;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.MulticastSocket;
import java.net.SocketException;
import java.net.UnknownHostException;

import java.io.IOException;

/**
 * This class encapsulates the CORBA::ORB singleton object.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.3
 */

public abstract class TheORBSingleton
{
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /**
     * To store the CORBA::ORB singleton object.
     */
    private static org.omg.CORBA.ORB orb_ = null;

    /**
     * To store initial object references.
     */
    private static java.util.Hashtable initialReferences_ =
        new java.util.Hashtable();
    // TODO: Must be removed when the operation
    // CORBA::ORB::register_initial_reference()
    // will be standardized by the OMG's ORB Core RTF.

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    protected TheORBSingleton()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     *  Initialize the ORB.
     *
     *  @param args The command-line arguments.
     *  @param orbClass The ORB class name.
     *  @param orbSingletonClass The ORB singleton class name.
     */
    protected static void initialize(
        String[] args,
        String orbClass,
        String orbSingletonClass)
    {

        java.util.Properties properties = System.getProperties();
        properties.put("org.omg.CORBA.ORBClass", orbClass);
        properties.put("org.omg.CORBA.ORBSingletonClass", orbSingletonClass);

        // Initialize the CORBA::ORB.
        setORB(org.omg.CORBA.ORB.init(args, properties));
    }

    /**
     * Requests an IOR from a multicast URL.
     *
     * @param location - The multicast group address of the IOR file.
     *
     * @return The IOR.
     */
    public static String
    request_ior(String location)
    {

        // Start a multicast client request

        String file =
                        location.substring(
                            location.indexOf("//") + 2).substring(
                            location.substring(
                                location.indexOf("//") + 2).indexOf(
                                "/")
                                + 1);
        String addr =
                        location.substring(
                            location.indexOf("//") + 2).substring(
                            0,
                            location.substring(
                                location.indexOf("//") + 2).indexOf(
                                "/"));

        // Define default used ports for sockets
        int s_port = 8888;
        int ms_port = 8880;

        // Sending request to server to get IOR

        InetAddress groupAddress = null;
        DatagramSocket socket;
        MulticastSocket msocket;

        try {
            groupAddress = InetAddress.getByName(addr);
        } catch (UnknownHostException ex) {
            System.out.println("Unknown host" + ex.getMessage());
        }

        String message = "NameService.IOR";

        // Send the name of the IOR file to request
        // default is NameService.IOR
        message = file;

        System.out.println("Sending multicast request to get IOR...");
        DatagramPacket send_request =
            new DatagramPacket(
                message.getBytes(),
                message.getBytes().length,
                groupAddress,
                ms_port);
        try {
            msocket = new MulticastSocket();
            msocket.send(send_request);
            msocket.disconnect();
            msocket.close();
            msocket.close();

        } catch (IOException e) {
            System.err.println(e.getMessage());
        }

        // Receiving response from server
        byte[] mem = new byte[1000];
        String ior_recv = null;
        try {
            socket = new DatagramSocket(s_port + 1);
            DatagramPacket recv = new DatagramPacket(mem, mem.length);
            socket.receive(recv);
            ior_recv = new String(mem, 0, recv.getLength() - 2);
            System.out.println(
                "Receiving IOR from server "
                    + recv.getAddress().getHostName()
                    + ":"
                    + recv.getPort());
            socket.disconnect();
            socket.close();
        } catch (SocketException e) {
        } catch (IOException e) {
        }

        return ior_recv;

    }

    /**
     * Remove ORB arguments.
     *
     * @param args The command-line arguments.
     *
     * @return The command line arguments without OpenORB options.
     */
    protected static String[]
    remove_ORB_arguments(String[] args) {
        int nb = 0;

        // Copy arguments without ORB ones.
        String[] tmp = new String[args.length];
        for (int i = 0; i < args.length; i++) {
            if (args[i].startsWith("-ORBInitRef")) {
                i++;
            } else if (!args[i].startsWith("-ORB")) {
                tmp[nb] = args[i];
                nb++;
            }
        }

        // Construct the returned array.
        args = new String[nb];
        for (int i = 0; i < nb; i++) {
            args[i] = tmp[i];
        }

        // Return arguments without ORB ones.
        return args;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain the CORBA::ORB singleton object.
     *
     * @return The CORBA::ORB singleton object.
     */
    public static org.omg.CORBA.ORB
    getORB()
    {
        if (orb_ == null) {
            orb_ = org.omg.CORBA.ORB.init();
        }
        return orb_;
    }

    /**
     * Set the CORBA::ORB singleton object.
     *
     * @param orb The CORBA::ORB singleton object.
     */
    public static void
    setORB(org.omg.CORBA.ORB orb)
    {
        orb_ = orb;
    }

    /**
     * Resolve a initial object reference.
     *
     * @param name The initial object name.
     *
     * @return The initial object reference.
     */
    public static org.omg.CORBA.Object
    resolve_initial_reference(String name)
    {
        // TODO: Must be removed when the operation
        // CORBA::ORB::register_initial_reference()
        // will be standardized by the OMG's ORB Core RTF.
        //
        Object object = initialReferences_.get(name);
        if (object != null) {
            return (org.omg.CORBA.Object) object;
        }

        try {
            return getORB().resolve_initial_references(name);
        } catch (org.omg.CORBA.ORBPackage.InvalidName exc) {
            // Wrap the CORBA::ORB::InvalidName exception.
            throw new UserExceptionWrapper(exc);
        }
    }

    /**
     * Register a initial object reference.
     *
     * @param name The initial object reference name.
     * @param object The initial object reference.
     */
    public static void
    register_initial_reference( String name,
                                org.omg.CORBA.Object object)
    {
        // TODO: Must be removed when the operation
        // CORBA::ORB::register_initial_reference()
        // will be standardized by the OMG's ORB Core RTF.
        //
        initialReferences_.put(name, object);

        // TODO: Must be replaced by
        //
        // getORB().register_initial_reference(name, object);
        //
        // when the operation CORBA::ORB::register_initial_reference()
        // will be standardized by the OMG's ORB Core RTF.
    }

    /**
     *  Run the CORBA::ORB singleton object.
     */
    public static void
    run()
    {
        getORB().run();
    }

    /**
     * Shutdown the CORBA::ORB singleton object.
     *
     * @param wait_for_completion
     *        Must (or not) wait for completion of current requests.
     */
    public static void
    shutdown(boolean wait_for_completion)
    {
        getORB().shutdown(wait_for_completion);
    }

    /**
     * Create a CORBA::Any.
     *
     * @return The created CORBA::Any.
     */
    public static org.omg.CORBA.Any
    create_any()
    {
        return getORB().create_any();
    }

    /**
     * Register a valuetype factory.
     *
     * @param id The valuetype repository Id.
     * @param factory The valuetype factory.
     */
    public static void register_value_factory(
        String id,
        org.omg.CORBA.portable.ValueFactory factory)
    {
        ((org.omg.CORBA_2_3.ORB) getORB()).register_value_factory(id, factory);
    }

    /**
     * Save an object reference into a file.
     *
     * @param object The object reference.
     * @param fileName The file name where the IOR will be stored.
     */
    public static void
    save_IOR(org.omg.CORBA.Object object, String fileName)
    {
        try {
            // Obtain the IOR of the object reference.
            String ior = getORB().object_to_string(object);

            // Instantiate the output file and stream.
            java.io.FileOutputStream file =
                new java.io.FileOutputStream(fileName);
            java.io.PrintWriter out = new java.io.PrintWriter(file);

            // Output the IOR.
            out.println(ior);

            // Flush the stream and close the file.
            out.flush();
            file.close();
        } catch (java.io.IOException exc) {
            // Wrap the exception.
            throw new java.lang.Error(exc.getMessage());
        }
    }

    /**
     * Convert a string to an object reference.
     *
     * @param string The stringified object reference.
     *
     * @return The object reference.
     */
    public static org.omg.CORBA.Object
    string_to_object(String string)
    {
        return getORB().string_to_object(string);
    }

    /**
     * Create the CORBA::VabueBase TypeCode.
     */
    public static org.omg.CORBA.TypeCode
    create_valuebase_tc()
    {
        return getORB().create_value_tc(
            "IDL:omg.org/CORBA/ValueBase:1.0",
            "ValueBase",
            org.omg.CORBA.VM_ABSTRACT.value,
            null,
            new org.omg.CORBA.ValueMember[0]);
    }

    /**
     * Obtain the null valuetype TypeCode.
     */
    public static org.omg.CORBA.TypeCode
    get_null_valuebase_tc()
    {
        return null;
    }
}
