/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import org.w3c.dom.Node;
import java.util.*;
import java.io.*;
import java.lang.reflect.*;

import org.omg.CORBA.ORB;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;

/**
 * The class IdlException represents an CORBA IDL exception value. Instances are created through one
 * of the create() factory methods in IdlNode.
 * An IdlException object can have child nodes representing exception member fields.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public class IdlException extends IdlNode implements IdlWritable {      
    protected String[] members;
    protected transient TypeCode[] types;

    protected IdlException() {
        isLeaf = false;
        isMutable = false;
        setUserObject(this);
    }
       
    public IdlException(Any any) {
        this();
        setNode(any);
    }
        
    protected IdlException(org.omg.CORBA.TypeCode tc) {
        this();
        setNode(tc);
    }

    protected void setNode(TypeCode tc) {
        try {
            this.tc = tc;  
            if (tc.kind().value() == TCKind._tk_except || tc.kind().value() == TCKind._tk_struct) {
                id = tc.id();
                type = "exception " + id;
                removeAllChildren();
                types = new TypeCode[tc.member_count()];
                members = new String[tc.member_count()];
                for (int i = 0; i < tc.member_count(); i++) {
                    types[i] = tc.member_type(i);
                    members[i] = tc.member_name(i);
                    IdlNode m = IdlNode.create(types[i]);
                    m.field = members[i];
                    add(m);                    
                }
            } else if (tc.kind().value() == TCKind._tk_alias) {
                id = tc.id();
                type = id;
                TypeCode stc = tc.content_type();
                while (stc.kind().value() == TCKind._tk_alias) {
                    stc = stc.content_type();
                }
                if (stc.kind().value() != TCKind._tk_except && stc.kind().value() != TCKind._tk_struct) {
                    throw new RuntimeException("Exception type expected");
                }

                removeAllChildren();
                types = new TypeCode[stc.member_count()];
                members = new String[stc.member_count()];
                for (int i = 0; i < stc.member_count(); i++) {
                    types[i] = stc.member_type(i);
                    members[i] = stc.member_name(i);
                    IdlNode m = IdlNode.create(types[i]);
                    m.field = members[i];
                    add(m);                     
                }
            } else {
                throw new RuntimeException("Exception type expected");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }
        
    protected void setNode(Any any) {
        try {           
            setNode(any.type());
            org.omg.CORBA.portable.InputStream in = any.create_input_stream();
            String xid = in.read_string();
            
            removeAllChildren();
            for (int i = 0; i < members.length; i++) {
                Any memberAny = orb.create_any();
                memberAny.read_value(in, types[i]);
                IdlNode m = create(memberAny);
                m.field = members[i];
                add(m);
            }
        } catch (Throwable e) {
            e.printStackTrace();
        }      
    }

    /**
     * Returns the list of exception member fields as a String array.
     *
     * @return A String array with all exception member field names.
     */
    public String[] getMembers() {
        return members;
    }
                
    /**
     * Returns the current value as a CORBA Any value.
     *
     * @return The current value as a CORBA Any.
     */
    public Any toAny() {
        try {           
            Any any = orb.create_any();
            any.type(tc);
            org.omg.CORBA.portable.OutputStream out = any.create_output_stream();
            out.write_string(id);
            for (int i = 0; i < members.length; i++) {
                ((IdlNode)getChildAt(i)).toAny().write_value(out);
            }
            any.read_value(out.create_input_stream(), tc);
            return any;
        } catch (Throwable e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Writes the value to a CORBA outputstream.
     *
     * @param is The outputstream to write to.
     */
    public void write(org.omg.CORBA.portable.OutputStream os) {
        try {
            for (int i = 0; i < members.length; i++) {
                ((IdlNode)getChildAt(i)).write(os);
            }                                    
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Reads the value from a CORBA inputstream.
     *
     * @param is The inputstream to read from.
     */
    public void read(org.omg.CORBA.portable.InputStream is) {
        try {
            for (int i = 0; i < members.length; i++) {
                ((IdlNode)getChildAt(i)).read(is);
            }                                    
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }
    
    // XML section

    /**
     * Create an IdlException from an Xml representation.
     *
     * The XML format example
     * <pre>
     * &lt; exception id="IDL:example/MyException:1.0"&gt;
     *  &lt;member name="message"&gt;
     *   &lt;string&gt; error message &lt;/string&gt;
     *  &lt;/member&gt;
     * &lt;/exception&gt;
     * </pre>
     * @param xml The XML string from which to create an IdlEnum instance.
     */
    public IdlException(String xml) {
        this(XmlNode.getNode(xml));
    }
    
    IdlException(Node n) {
        this();
        try {
            id = XmlNode.getId(n);
            setNode(XmlNode.type(id));
           
            Node[] nodes = XmlNode.childElements(n);
            String[] xmlmembers = new String[nodes.length];

            for (int i = 0; i < nodes.length; i++) {
                if (nodes[i].getNodeName().toUpperCase().equals("MEMBER")) {
                    xmlmembers[i] = XmlNode.getName(nodes[i]);
                    IdlNode m = XmlNode.getIdlNode(tc.member_type(i), XmlNode.firstChildElement(nodes[i]));
                    m.setField(xmlmembers[i]);
                    add(m);
                } else {
                    throw new RuntimeException("Member expected.");
                }
            }      
            if (members.length >= xmlmembers.length) {
                for (int i = 0; i < members.length; i++) {
                    if (i >= xmlmembers.length || !members[i].equals(xmlmembers[i])) {
                        throw new RuntimeException("exception member " + members[i] + " is missing for " + id);
                    }
                }
            } else {
                for (int i = 0; i < xmlmembers.length; i++) {
                    if (i >= members.length || !members[i].equals(xmlmembers[i])) {
                        throw new RuntimeException(xmlmembers[i] + " is not a valid exception member in " + id);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
        
    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        write(this, w);
    }

    public static void write(IdlException n, IdlWriter w) {
        w.write_start_exception(n.getId());
        String[] mbrs = n.getMembers();
        for(int i = 0; i < n.getChildCount(); i++) {
            w.write_start_member(mbrs[i]);
            XmlNode.write((IdlNode)n.getChildAt(i), w);
            w.write_end_member();
        }        
        w.write_end_exception();
    }
}