/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import org.w3c.dom.Node;
import javax.swing.tree.*;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;
import org.omg.CORBA.ORB;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;

/**
 * The class IdlSequence represents an CORBA IDL sequence value. Instances are created through one
 * of the create() factory methods in IdlNode.
 * An IdlSequence object has child nodes representing the content of the IDL sequence it represents. The type and
 * value of the child node correspond with the type and value of the elements in the sequence it represents.
 * The sequence length and element type is extracted from the typecode value for the sequence during initialization.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public class IdlSequence extends IdlNode implements IdlWritable {  
    protected boolean isBounded;
    protected int size;
    protected transient TypeCode content;
    
    protected IdlSequence() {
        isLeaf = false;
        setUserObject(this);
    }
   
    protected IdlSequence(Any any) {
        this();
        setNode(any);
    }
        
    protected IdlSequence(TypeCode tc) {
        this();
        setNode(tc);
    }
    
    protected void setNode(TypeCode tc) {
        try {
            this.tc = tc;
            if (tc.kind().value() == TCKind._tk_sequence) {
                id = "";
                try {
                    id = tc.id();
                    type = "sequence " + id;
                } catch (Exception ex) {
                    // anonymous sequences will cause an exception
                    type = "sequence";
                }
                if (tc.length() == 0) {
                    isBounded = false;
                } else {
                    isBounded = true;
                    size = tc.length();
                }
                content = tc.content_type();
            } else if (tc.kind().value() == TCKind._tk_alias) {
                id = tc.id();
                type = id;
                TypeCode stc = tc.content_type();
                while (stc.kind().value() == TCKind._tk_alias) {
                    stc = stc.content_type();
                }
                if (stc.kind().value() != TCKind._tk_sequence) {
                    throw new RuntimeException("Sequence type expected");
                }
                if (stc.length() == 0) {
                    isBounded = false;
                } else {
                    isBounded = true;
                    size = stc.length();
                }
                content = stc.content_type();
            } else {
                throw new RuntimeException("Sequence type expected");
            } 
            removeAllChildren();
            if (size == 0) {
                isBounded = false;
                size = 1;
                value = "1";
                // for unbound sequences create one default element
                IdlNode m = create(content);
                m.field = "[0]";
                add(m);
            } else {
                isBounded = true;
                value = "" + size;
                for (int i = 0; i < size; i++) {
                    IdlNode m = create(content);
                    m.field = "[" + i + "]";
                    add(m);
                }
            }                                         
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }

    protected void setNode(Any any) {
        try {
            setNode(any.type());
            org.omg.CORBA.portable.InputStream in = any.create_input_stream();
            if (isBounded) {
                int receivedSize = in.read_ulong();
                if (receivedSize > size) {
                    throw new RuntimeException("Unexpected sequence size " + receivedSize + ". " + id + " has a maximum size of " + size + ".");
                }
            } else {
                size = in.read_ulong();
            }
            removeAllChildren();
            for (int i = 0; i < size; i++) {
                Any memberAny = orb.create_any();
                memberAny.read_value(in, content);
                IdlNode m = create(memberAny);
                m.field = "[" + i + "]";
                add(m);
            }
            value = "" + size;                                     
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }
    
    /**
     * Tests if the sequence length is bound to a maximum value.
     *
     * @return True if the sequence length is bound to a maximum value, false otherwise.
     */
    public boolean isBounded() {
        return isBounded;
    }
    
    /**
     * Returns the number of sequence elements.
     *
     * @return The number of sequence elements.
     */
    public int getSize() {
        return size;
    }
    
    /**
     * Sets the number of sequence elements.
     *
     * @param v The number of sequence elements.
     */
     public void setValue(String v) {
        try {
            setSize(Integer.parseInt(v));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }     

    /**
     * Sets the number of sequence elements.
     *
     * @param s The number of sequence elements.
     */
    public void setSize(int s) {
        try {
            if (isBounded && s > tc.length()) {
                throw new RuntimeException("Unexpected sequence size " + s + ". " + id + " has a maximum size of " + tc.length() + ".");
            }
            removeAllChildren();
            size = s;
            value = "" + size;
            for (int i = 0; i < size; i++) {
                IdlNode m = create(content);
                m.field = "[" + i + "]";
                add(m);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Returns the current value as a CORBA Any value.
     *
     * @return The current value as a CORBA Any.
     */
    public Any toAny() {
        try {
            Any any = orb.create_any();
            any.type(tc);
            org.omg.CORBA.portable.OutputStream out = any.create_output_stream();
            out.write_ulong(size);
            for (int i = 0; i < size; i++) {
                ((IdlNode)getChildAt(i)).toAny().write_value(out);
            }
            any.read_value(out.create_input_stream(), tc);
            return any;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Writes the value to a CORBA outputstream.
     *
     * @param is The outputstream to write to.
     */
    public void write(org.omg.CORBA.portable.OutputStream os) {
        try {
            for (int i = 0; i < size; i++) {
                ((IdlNode)getChildAt(i)).write(os);
            }                                    
        } catch (Exception e) {
            e.printStackTrace();
        }  
    }

    /**
     * Reads the value from a CORBA inputstream.
     *
     * @param is The inputstream to read from.
     */
    public void read(org.omg.CORBA.portable.InputStream is) {
        try {
            for (int i = 0; i < size; i++) {
                ((IdlNode)getChildAt(i)).read(is);
            }                                    
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }
    
    // XML section

    public IdlSequence(String xml) {
        this(XmlNode.getNode(xml));
    }

    IdlSequence(Node n) {
        this(null, n);        
    }
    
    IdlSequence(org.omg.CORBA.TypeCode t, Node n) {
        this();
        try {
            id = XmlNode.getId(n);
            if (!id.equals("")) {
                t = XmlNode.type(id);
            }
            setNode(t);
            
            if (tc.kind().value() == TCKind._tk_alias) {
                org.omg.CORBA.TypeCode content = tc.content_type().content_type();
                size = tc.content_type().length();
            } else {
                org.omg.CORBA.TypeCode content = tc.content_type();
                size = tc.length();
            }            
            int length = XmlNode.getLength(n); 

            if (size != 0 && size < length) {
                throw new RuntimeException("This sequence length has a maximum of " + size + " elements.");
            }

            Node[] nodes = XmlNode.childElements(n);
    
            if (nodes.length != length) {
                throw new RuntimeException("Number of sequence elements does not match " + length + ".");
            }
            removeAllChildren();
            for (int i = 0; i < length; i++) {
                IdlNode m = XmlNode.getIdlNode(content, nodes[i]);
                m.setField(field + "[" + i + "]");
                add(m);
            }
            value = "" + length;
        } catch (Exception e) {
e.printStackTrace();
//            throw new RuntimeException(e.toString());
        }
    }    

    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        write(this, w);
    }

    public static void write(IdlSequence n, IdlWriter w) {
        w.write_start_sequence(n.getId(), n.getSize());
        for(int i = 0; i < n.getChildCount(); i++) {
            XmlNode.write((IdlNode)n.getChildAt(i), w);
        }        
        w.write_end_sequence();
    }
}