/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base;

//Package dependencies.
import java.io.StringWriter;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import org.objectweb.apollon.framework.Bean;

/**
 * The DeployerContext class is a basic deployer context in charge to provide
 * basis methos usefull for the deployment. 
 *
 * @author <a href="mailto:briclet@lifl.fr">Frederic Briclet</a>
 *
 * @version 0.1
 */
public abstract class DeployerContext 
    implements Deployer 
{

    // ==================================================================
    //
    // Internal state.
    //
    // ===================================================================
    //The current life cycle manager
    private LifeCycleManager lifeCycleManager;
    //The current deployer scheduler
    private DeployerScheduler deployerScheduler;
    //The current error manager
    private ErrorManager errorManager;
    //The bean description of the xml code processed
    private Bean deployerDescription;
    //The bean stringifiedDescription 
    private String stringifiedDescription = null;

    //  ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    //  ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    //  ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================
    /**
     * The getRootDeployerContext is an abstract method which must be implemented
     * by children because the root deployer context will support differently this 
     * method than another deployer.
     * @return The RootDeployerContext of this deployment
     */
    public abstract RootDeployerContext getRootDeployerContext();

    /**
     * The visit method is in charge to execute the deployment of  the
     * deployer given in parameter. In this generic implementation the
     * reflexion is used to invoke the right method in charge to process
     * the given @param depl Deployer. The current deployer must provide
     * a "visit" method that take in parameter the @param depl deployer kind
     * and the current deployer status. The method signature is the following :
     * void visit(@param depl_type , CurrentDeployerState);      
     * 
     * @Param depl the deployer to process for deployment execution
     * @FatalDeploymentException if no right visit method is found or if the
     * deployer execution thrown it. 
     */
    public void visit(Deployer depl) throws FatalDeploymentException {
        try {
            Method m =
                this.getClass().getDeclaredMethod(
                    "visit",
                    new Class[] {
                        depl.getClass(),
                        this.getStatus().getClass()});
            m.invoke(this, new Object[] { depl, this.getStatus()});

        } catch (NoSuchMethodException e) {
            //System error
            throw new FatalDeploymentException
                        (e,"Deployment architecture corrupted check the deployer "+
                           "implementation it segurely miss a visit method in right state");
        } catch (IllegalAccessException e1) {
            throw new FatalDeploymentException
                        (e1,"Deployment architecture corrupted");
        } catch (InvocationTargetException e2) {
            if(e2.getTargetException() instanceof FatalDeploymentException)
                throw (FatalDeploymentException)e2.getTargetException();
            throw new FatalDeploymentException
                        ((Exception) e2.getTargetException(),
                          "Deployment architecture throw an unexpected exception");
        }
    }

    /**
     * The traverse method make the scheduling of the deployers
     * processing.The scheduling is delegated to the deployment 
     * scheduler which is connected.
     * 
     * @param deployer The deployer in charge to process @param deployers
     * @param deployers The deployers to process
     * @throws FatalDeploymentException if a problem occured during the 
     * deployment or if the the given @param deployers are not supported
     * by @param deployer.
     */
    public void 
    traverse(Deployer deployer, Deployer[] deployers)
    throws FatalDeploymentException 
    {
        deployerScheduler.traverse(deployer, deployers);
    }

    /**
     * The connectLifeCycleManager method connect a Life Cycle Manager to 
     * the current deployer. This life cycle manager will be used to memorize
     * the deployer state.
     * 
     * @param lifeCycleManager The LifeCycleManager to connect
     */
    public void
    connectLifeCycleManager(LifeCycleManager lifeCycleManager) 
    {
        this.lifeCycleManager = lifeCycleManager;
    }
    
    /**
     * Accessor method to retrieve the current Life Cycle Manager
     * @return The current LifeCycleManager
     */
    public LifeCycleManager 
    getLifeCycleManager()
    {
        return this.lifeCycleManager;
    }
    

    /**
     * The connectErrorManager method connect a Error Manager to the
     * current deployer
     * 
     * @param manager The ErrorManager to connect
     */
    public void 
    connectErrorManager(ErrorManager manager)
    {
        errorManager = manager;
    }
           
    /**
     * Method accesor to retrieve the current Error Manager
     * 
     * @return The current error manager
     */
    public ErrorManager 
    getErrorManager() 
    {
        return errorManager;
    }
    
    /**
     * The connectDeployerScheduler method is used to connect and 
     * deployer scheduler to the current deployer.
     * 
     * @param deployerScheduler The DeployerScheduler to connect
     */
    public void 
    connectDeployerScheduler(DeployerScheduler deployerScheduler)
    {
        this.deployerScheduler = deployerScheduler;
    }

    /**
     * The connectDeployerDescription is used to connect the xml element
     * which describe the xml code processed. 
     * @param deployerDescription to connect.
     */
    public void 
    connectDeployerDescription(Bean deployerDescription)
    {
        this.deployerDescription = deployerDescription;
    }
       
    /**
     * The getStatus method return the current deployer state. 
     * In this implementation the deployer state is managed by an
     * external life cycle manager. The getStatus implementation
     * processing is delegate to the current LifeCycleManager.
     * 
     * @return DeployerState The current deployer state
     */
    public DeployerState 
    getStatus()
    {
        return lifeCycleManager.getStatus();
    }


    /**
     * The getStringifiedDeployerDescription method provide a
     * readable string reprensentation of the xml code process
     * by the current deployer.
     * 
     * @return The stringified representation of the code processed
     * @throws InitializationError thrown if the deployerDescription is
     * not connected.
     */
    public String 
    getStringifiedDeployerDescription()
    throws InitializationError 
    {
        try {
            if (stringifiedDescription == null) {
                StringWriter w = new StringWriter();
                deployerDescription.marshal(w);
                stringifiedDescription = w.toString();
            }
            return stringifiedDescription;
        } catch (Exception e) {
            e.printStackTrace();
            throw new InitializationError
                        (this,
                            "The deployer description retrieval failed due to:\n"
                                + e.getMessage());
        }
    }
    /**
     * The getStringifiedDeployerDescriptionNoError method provide a
     * readable string reprensentation of the xml code process
     * by the current deployer.
     * 
     * @return The stringified representation of the code processed
     */
    public String 
    getStringifiedDeployerDescriptionNoError()
    {
        try {
            if (stringifiedDescription == null) {
                StringWriter w = new StringWriter();
                deployerDescription.marshal(w);
                stringifiedDescription = w.toString();
            }
            return stringifiedDescription;
        } catch (Exception e) {
            return "Description not available";
        }
    }
    /**
     * The getIdentifier method provides a string representation of
     * the current deployer. This string representation is compute
     * using the Depoyer class name.
     */
    public String 
    getIdentifier() 
    {
        String name = this.getClass().getName();
        return name.substring(name.lastIndexOf(".") + 1);
    }
    
    /**
     * The abstract initialize method must be implemented by the child
     * deployer context to verify that all the required connection for
     * deployment processing are established. And all the xml code provided
     * is valide. a successfull invokation of the initialize method chance 
     * the current deployer state to the following
     * (classicaly the DeployerInactiveState). 
     * 
     * @throws InitializationError thrown if a connection required missed or if
     * a data is not well feed or formated.
     */
    public abstract void 
    initialize() 
    throws InitializationError;


}
