/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.cif_idl.lib;

// Package dependencies.
import org.objectweb.corba.ast.api.ExecutorDecl;
import org.objectweb.corba.ast.api.ComponentDecl;
import org.objectweb.corba.ast.api.InterfacePortDecl;
import org.objectweb.corba.ast.api.SegmentDecl;
import org.objectweb.corba.ast.api.EventPortDecl;
import org.objectweb.corba.ast.api.InterfaceList;
import org.objectweb.corba.ast.api.NativeDecl;
import org.objectweb.corba.ast.api.OperationDecl;
import org.objectweb.corba.ast.api.InterfaceDecl;
import org.objectweb.corba.ast.api.ModuleDecl;
import org.objectweb.corba.ast.api.Declaration;
import org.objectweb.corba.ast.api.CategoryKind;
import org.objectweb.corba.ast.utils.api.CompositionInfo;
import org.objectweb.corba.ast.utils.api.ExecutorInfo;
import org.objectweb.corba.generator.cif_idl.api.SegmentMapping;
import java.util.Set;


/**
 * This class generates CIF IDL for an OMG CIDL Component Executor.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class ExecutorMapping
  implements org.objectweb.corba.generator.cif_idl.api.ExecutorMapping
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The component executor to map. */
    private ExecutorDecl exec_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * @param exec - The executor to map.
     */
    public ExecutorMapping(ExecutorDecl exec)
    {
        // Init internal states
        exec_ = exec;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Add Component port interface mapping to the Interface List.
     *
     * @param composition - An abstract view of the binding composition.
     * @param list - The list to add port mapping in.
     */
    protected void
    addInterfacePortMapping(CompositionInfo composition,
                            InterfaceList list)
    {
        ExecutorInfo exec_info = null;
        Declaration[] implemented_facets = null;
        Set itf_set = null;

        itf_set = new java.util.HashSet();
        exec_info = new org.objectweb.corba.ast.utils.lib.ExecutorInfo(exec_);
        implemented_facets = exec_info.getMainSegmentFacets();

        // Add port mapping interface to the set
        for (int i=0; i<implemented_facets.length; i++)
        {
            try {
                InterfacePortDecl itf_port = null;

                itf_port = (InterfacePortDecl) implemented_facets[i];
                itf_set.add( itf_port.getLocalMapping() );
            }catch(ClassCastException e){
                EventPortDecl event_port = null;

                event_port = (EventPortDecl) implemented_facets[i];
                itf_set.add( event_port.getEvent().getLocalConsumerMapping() );
            }
        }

        // Add all interfaces to the list
        java.util.Iterator it = itf_set.iterator();
        while ( it.hasNext() )
        {
            list.add((InterfaceDecl) it.next());
        }
    }

    /**
     * Create Executor CIF interface mapping.
     *
     * @param composition - An abstract view of the binding composition.
     * @param module - The module to declare CIF mapping in.
     * @param seg_base_mapping - The Segment base CIF interface mapping.
     *
     * @return The executor mapping.
     */
    protected InterfaceDecl
    create_executor_mapping(CompositionInfo composition,
                            ModuleDecl module,
                            InterfaceDecl seg_base_mapping)
    {
        InterfaceDecl mapping = null,
                      tmp = null;
        InterfaceList list = null;
        CategoryKind kind = null;


        mapping = module.declareLocalInterface( "CIF_" + exec_.getName() );

        // Feed inherited interface list
        list = mapping.getInheritedInterfaceList();

        // Add the CIF_SegmentBase interface
        list.add(seg_base_mapping);

        // Add the ExecutorLocator interface
        tmp = (InterfaceDecl) exec_.getAST().lookup("::Components::ExecutorLocator");
        list.add(tmp);

        // Add the <kind>Component interface
        kind = exec_.getCompositionKind();
        if (    (kind == CategoryKind.ck_entity)
             || (kind == CategoryKind.ck_process) )
        {
            tmp = (InterfaceDecl) exec_.getAST().lookup("::Components::EntityComponent");
        }
        else
        {
            tmp = (InterfaceDecl) exec_.getAST().lookup("::Components::SessionComponent");
        }
        list.add(tmp);

        // Add the CCM_<component_name>_Executor interface (segmented strategy)
        tmp = composition.getComponent().getLocalMainMapping();
        list.add(tmp);

        // Add facets and sinks directly implemented by the component executor
        addInterfacePortMapping(composition, list);

        // Create the mapping
        mapping.create();

        return mapping;
    }

    /**
     * Add Executor CIF interface mapping contents.
     *
     * @param composition - An abstract view of the binding composition.
     * @param mapping - The Executor CIF interface mapping.
     * @param ast - The Executor Abstract Storage Type.
     */
    protected void
    add_executor_mapping_contents(InterfaceDecl mapping,
                                  NativeDecl ast)
    {
        if (ast != null)
        {
            OperationDecl op = null;

           // create the get_state operation
            op = mapping.startOperation("get_state");
            op.setType(ast);
            op.create();
        }

    }

    /**
     * Create Segment Base CIF interface mapping.
     *
     * @param composition - An abstract view of the binding composition.
     *
     * @return The segment base mapping.
     */
    protected InterfaceDecl
    create_seg_base_mapping(ModuleDecl module)
    {
        InterfaceDecl seg_base_mapping = null;

        seg_base_mapping = module.declareLocalInterface("CIF_SegmentBase");
        seg_base_mapping.create();

        return seg_base_mapping;
    }

    /**
     * Add Segment Base CIF interface mapping contents.
     *
     * @param seg_base_mapping - The Segment Base CIF interface mapping.
     * @param exec_mapping - The Executor CIF interface mapping.
     * @param composition - An abstract view of the binding composition.
     */
    protected void
    add_seg_base_mapping_contents(InterfaceDecl seg_base_mapping,
                                  InterfaceDecl exec_mapping,
                                  CompositionInfo composition)
    {
        OperationDecl op = null;

        op = seg_base_mapping.startOperation("get_context");
        // Retrieve CCM_<component_name>_Context interface
        op.setType(composition.getComponent().getLocalContextMapping());
        op.create();

        op = seg_base_mapping.startOperation("get_main_segment");
        op.setType(exec_mapping);
        op.create();
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.generator.cif_idl.api.ExecutorMapping
    //
    // ==================================================================

    /**
     * Declare CIF interfaces into Interface Repository.
     *
     * @param composition - An abstract view of the binding composition.
     * @param module - The module to declare CIF mapping in.
     * @param ast - The Executor Abstract Storage Type.
     */
    public void
    declareCIF( CompositionInfo composition,
                ModuleDecl module,
                NativeDecl ast)
    {
        ComponentDecl comp = null;
        SegmentDecl[] segments = null;
        InterfaceDecl seg_base_mapping = null,
                      exec_mapping = null;
        SegmentMapping seg_mapping = null;

        // Get the managed component
        comp = composition.getComponent();

        /** Create the segment base CIF interface **/
        seg_base_mapping = create_seg_base_mapping(module);

        /** Create the executor CIF interface **/
        exec_mapping = create_executor_mapping(composition,
                                               module,
                                               seg_base_mapping);
        add_executor_mapping_contents(exec_mapping, ast);

        /** Finish creation of the segment base interface **/
        add_seg_base_mapping_contents(seg_base_mapping,
                                      exec_mapping,
                                      composition);

        /** Call mapping for segments **/
        segments = composition.getSegments();
        for (int i=0; i<segments.length; i++)
        {
            seg_mapping = new org.objectweb.corba.generator.cif_idl.lib.SegmentMapping(segments[i]);
            seg_mapping.declareCIF( module,
                                    seg_base_mapping,
                                    comp );
        }
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
