/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.api;

// Package dependencies.
import java.io.PrintStream;

/**
 * This interface wraps standard output and error streams.
 *
 * This provides the following properties:
 *
 * - OutputStream: The standard output stream.
 *
 * - ErrorStream: The standard error stream.
 *
 * - MessageHeader: The header printed before messages.
 *
 * - SilentMessage: Flag to indicate if messages will be really printed or not.
 *
 * This provides the following methods:
 *
 * - message: Prints messages to the standard output stream.
 *
 * - error:   Prints messages to the standard error stream.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public interface Console
{
    /**
     * Gets the standard output stream.
     *
     * @return The standard output stream.
     */
    public PrintStream
    getOutputStream();

    /**
     * Sets the standard output stream.
     *
     * @param output The standard output stream.
     */
    public void
    setOutputStream(PrintStream output);

    /**
     * Gets the standard error stream.
     *
     * @return The standard error stream.
     */
    public PrintStream
    getErrorStream();

    /**
     * Sets the standard error stream.
     *
     * @param error The standard error stream.
     */
    public void
    setErrorStream(PrintStream error);

    /**
     * Gets the message header.
     *
     * @return The message header.
     */
    public String
    getMessageHeader();

    /**
     * Sets the message header.
     *
     * @param header The message header.
     */
    public void
    setMessageHeader(String header);

    /**
     * Gets the silent flag status.
     *
     * @return The silent flag status.
     */
    public boolean
    getSilentMessage();

    /**
     * Sets the silent flag status.
     *
     * @param flag The silent flag status.
     */
    public void
    setSilentMessage(boolean flag);

    /**
     * Prints a message on the standard output.
     *
     * @param text The message to print.
     */
    public void
    message(String text);

    /**
     * Displays a message on the error output.
     *
     * @param text The message to print.
     */
    public void
    error(String text);
}
