/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.openccm.ast.lib;

// Package dependencies.
import org.objectweb.openccm.ast.api.DeclarationKind;
import org.objectweb.openccm.ast.api.Declaration;
import org.objectweb.openccm.ast.api.DelegationList;
import org.objectweb.openccm.ast.api.StringList;
import org.objectweb.openccm.ast.api.HomeDecl;
import org.objectweb.openccm.ast.api.AbstractStorageHomeDecl;
import org.objectweb.openccm.ast.api.ExecutorDecl;

/**
 * HomeExecutorDeclImpl is a wrapper class for CIDL home executor declarations.
 *
 *
 * Inherits from:
 * - CidlScopeImpl as executors are also IDL scopes.
 * - HomeExecutorDecl as OMG IDL for CIDL home executor declarations.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.2
 */

public class HomeExecutorDeclImpl
     extends CidlScopeImpl
  implements org.objectweb.openccm.ast.api.HomeExecutorDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated home declaration. */
    private HomeDeclImpl home_;

    /** The associated abstract storagehome binding. */
    private AbstractStorageHomeDeclImpl ashBinding_;

    /** The associated abstract storagehome persistence. */
    private AbstractStorageHomeDeclImpl ashPersistence_;

    /** List of abstract storagehome delegations. */
    private DelegationListImpl ash_delegation_;

    /** List of executor delegations. */
    private DelegationListImpl executor_delegation_;

    /** List of abstract operations. */
    private StringListImpl operations_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the exception declaration.
     */
    public
    HomeExecutorDeclImpl(Repository rep,
                         ScopeImpl parent)
    {
        // Call the ScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        home_ = null;
        ashBinding_ = null;
        ashPersistence_ = null;
        ash_delegation_ = new DelegationListImpl();
        executor_delegation_ = new DelegationListImpl();
        operations_ = new StringListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the exception
     *         declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the exception
     *          declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
        return null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_ != null)
            return dependencies_;

        java.util.List he_depend = new java.util.ArrayList();
        Declaration[] depend = null;

        // implemented Home
        if ( he_depend.indexOf(getHomeType()) == -1 )
            he_depend.add( getHomeType() );
        depend = getHomeType().getDependencies();
        for (int j=0; j<depend.length; j++)
        {
            if ( (depend[j] != this) &&
                 (he_depend.indexOf(depend[j]) == -1) )
                he_depend.add(depend[j]);
        }

        // Managed component
        he_depend.add( getHomeType().getManagedComponent() );
        depend = getHomeType().getManagedComponent().getDependencies();
        for (int j=0; j<depend.length; j++)
        {
            if ( (depend[j] != this) &&
                 (he_depend.indexOf(depend[j]) == -1) )
                he_depend.add(depend[j]);
        }

        // Abstract Storage Home binding
        if (getAbstractStorageHomeBinding() != null)
        {
            he_depend.add( getAbstractStorageHomeBinding() );
            depend = getAbstractStorageHomeBinding().getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if ( (depend[j] != this) &&
                     (he_depend.indexOf(depend[j]) == -1) )
                    he_depend.add(depend[j]);
            }
        }

        // Home persistence
        if (getHomePersistence() != null)
        {
            he_depend.add( getHomePersistence() );
            depend = getHomePersistence().getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if ( (depend[j] != this) &&
                     (he_depend.indexOf(depend[j]) == -1) )
                    he_depend.add(depend[j]);
            }
        }

        // Abstract Storage Home delegation
        // TO DO

        // Executor delegation
        // TO DO

        // contents
        Declaration[] decls = getContents(true, DeclarationKind.dk_all);
        for (int i=0; i<decls.length; i++)
        {
            depend = decls[i].getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if ( (!containsDecl(depend[j])) &&
                     (depend[j] != this) &&
                     (he_depend.indexOf(depend[j]) == -1) )
                {
                    he_depend.add(depend[j]);
                }
            }
        }

        dependencies_ = (Declaration[])he_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_home_executor;
    }

    /**
     * Close the exception scope.
     */
    public void
    create()
    {
        // Call the Cidl scope method
        super.create();

      // TODO check if the home executor declaration is correct
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.HomeExecutorDecl
    //
    // ==================================================================

    /**
     * Set the associated home declaration.
     *
     * @param ash The associated home declaration.
     */
    public void
    setHomeType(HomeDecl h)
    {
        home_ = (HomeDeclImpl)h;
    }

    /**
     * Obtain the associated home declaration.
     *
     * @return The associated home declaration.
     */
    public HomeDecl
    getHomeType()
    {
        return home_;
    }

    /**
     * Set the associated binding abstract storagehome.
     *
     * @param ash The associated binding abstract storagehome.
     */
    public void
    setAbstractStorageHomeBinding(AbstractStorageHomeDecl ash)
    {
        ashBinding_ = (AbstractStorageHomeDeclImpl)ash;
    }

    /**
     * Obtain the associated binding abstract storagehome.
     *
     * @return The associated binding abstract storagehome.
     */
    public AbstractStorageHomeDecl
    getAbstractStorageHomeBinding()
    {
        return ashBinding_;
    }

    /**
     * Set the associated persistence abstract storagehome.
     *
     * @param ash The associated persistence abstract storagehome.
     */
    public void
    setHomePersistence(AbstractStorageHomeDecl ash)
    {
        ashPersistence_ = (AbstractStorageHomeDeclImpl)ash;
    }

    /**
     * Obtain the associated persistence abstract storagehome.
     *
     * @return The associated persistence abstract storagehome.
     */
    public AbstractStorageHomeDecl
    getHomePersistence()
    {
        return ashPersistence_;
    }

    /**
     * Obtain the list of abstract storagehome delegations.
     *
     * @return The list of abstract storagehome delegations.
     */
    public DelegationList
    getAbstractStorageHomeDelegationList()
    {
        return ash_delegation_;
    }

    /**
     * Obtain the list of executor delegations.
     *
     * @return The list of executor delegations.
     */
    public DelegationList
    getExecutorDelegationList()
    {
        return executor_delegation_;
    }

    /**
     * Obtain the list of abstract operations.
     *
     * @return The list of abstract operations.
     */
    public StringList
    getAbstractOperationList()
    {
        return operations_;
    }

    /**
     * Start a new CIDL executor declaration.
     *
     * @param name The name of the CIDL executor declaration.
     *
     * @return The new created ExecutorDecl.
     */
    public ExecutorDecl
    startExecutor(String name)
    {
        ExecutorDecl decl = new ExecutorDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }
}
