# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
# Contact: openccm@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Pierre Carpentier.
# Contributor(s): ______________________________________.
#
# --------------------------------------------------------------------
# $Id: ASTFeeder.cs,v 1.1 2004/04/15 09:09:01 carpentier Exp $
# ====================================================================
#
#  This class feeds the AST from the differents elements in the
#  UML repository.
#
# ====================================================================

import NotCreatedException;
import UMLOperations;

class ASTFeeder
{

    # =================================================================
    # Empty Constructor for the AST feeder.
    # =================================================================
    proc __ASTFeeder__(self)
    {
        self.umlOperations = UMLOperations.UMLOperations();
        self._types = {};
        self._to_generate = {};
        self._generated_module = array.create(0);

        JAVA.declare("org.objectweb.openccm.ast.lib");
        JAVA.declare("org.objectweb.openccm.ccm");
        JAVA.declare("org.objectweb.openccm.corba");
        JAVA.declare("java.util");

        ir = org.objectweb.openccm.corba.TheInterfaceRepository.getRepository();
        self.ast = org.objectweb.openccm.ast.lib.Repository(org.objectweb.openccm.ccm.ComponentRepositoryHelper.narrow(ir));

        self.scope = self.ast.startFileScope("");
        self.scope.create();
        self.prefix = "";
        self.moduleName = "no-module";
    }


    # =================================================================
    # Gets the name of the root module.
    # =================================================================
    proc getModuleName (self)
    {
        return self.moduleName;
    }


    # =================================================================
    # Puts the type in the hashtable.
    # element - The UML element corresponding to the type
    # type - The type itself
    # =================================================================
    proc putType (self, element, type)
    {
        self._types[element] = type;
    }


    # =================================================================
    # Gets the type from the hashtable. If the type is not present in
    # the hashtable, the method throws the NotCreatedException.
    # element - The UML element corresponding to the type
    # return - The type itself
    # =================================================================
    proc getType (self, element)
    {
        if (!self.typeExist(element)) {
            throw NotCreatedException.NotCreatedException(element.getName());
        }
        return self._types[element];
    }


    # =================================================================
    # Tests if the type is present in the hashtable.
    # element - The UML element corresponding to the type
    # return - true if the type is present, else false
    # =================================================================
    proc typeExist (self, element)
    {
        return self._types.containsKey(element);
    }


    # =================================================================
    # Sets the element as generated.
    # element - The UML element
    # =================================================================
    proc setGenerated (self, element)
    {
        if (self._to_generate.containsKey(element))
            self._to_generate.remove(element);
    }


    # =================================================================
    # Feeds the Interface Repository from the UML model.
    # umlPackage - The UML Package
    # =================================================================
    proc createModel(self)
    {
        JAVA.declare("org.objectweb.openccm.uml.lib");
        self.umlPackage = org.objectweb.openccm.uml.lib.TheUMLPackage.getUMLPackage();
        self.umlOperations.setUMLPackage(self.umlPackage);
        self.undefinedType = false;
        try {
            packages = self.umlPackage.getModel_Management().getPackage().refAllOfType().toArray();
            self.create(packages, self.scope);
            while (self._to_generate.size > 0) {
                oldSize = self._to_generate.size;
                for element in self._to_generate.keys {
                    self.createElement(element, self._to_generate[element]);
                }
                if (oldSize == self._to_generate.size) {
                    self.undefinedType = true;
                }
            }
        } catch (exception) {
            for module in self._generated_module {
                module.destroy();
            }
            throw exception;
        }
    }


    # =================================================================
    # Create some elements of the model into the Interface Repository.
    # modelElements - The elements
    # currentScope - The current scope in which create the element
    # =================================================================
    proc create (self, modelElements, currentScope)
    {
        for element in modelElements {
            self.createElement(element, currentScope);
        }
    }


    # =================================================================
    # Create one element of the model into the Interface Repository.
    # element - The element
    # currentScope - The current scope in which create the element
    # =================================================================
    proc createElement (self, element, currentScope)
    {
        stereotypes = element.getStereotype().toArray();
        if (stereotypes.length == 0) {
            self.createContents(element, currentScope);
        } else {
            try {
                for stereotype in stereotypes {
                    eval("self.create" + stereotype.getName() + "(element, currentScope);");
                }
                self.setGenerated(element);
            } catch (NotCreatedException.NotCreatedException nce) {
                self._to_generate[element] = currentScope;
                if (self.undefinedType)
                    throw nce;
            } catch (NotFound nf) {
                self.createContents(element, currentScope);
            }
        }
    }

    proc createContents (self, element, currentScope) {
        if (element._is_a(org.omg.uml.core.UmlAttribute)) {
            self.createAttribute(element, currentScope);
        } else if (element._is_a(org.omg.uml.core.Operation)) {
            self.createOperation(element, currentScope)
        } else if (element._is_a(org.omg.uml.core.UmlClass)) {
            # Some UML Tools put Exception into the Exception class.
            for ownedEl in element.getOwnedElement().toArray() {
                if (ownedEl._is_a(org.omg.uml.common_behavior.UmlException)) {
                    self.createElement(ownedEl, currentScope);
                }
            }
        }
    }

}
