/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: XMI2IR.java,v 1.1 2004/04/15 09:09:01 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.lib;

import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.openccm.ast.api.FileScope;

/**
 * Implementation of the xmi2ir command.
 *
 * @author <a href="mailto:Pierre.Carpentier@lifl.fr">Pierre Carpentier</a>
 *
 * @version 1.0
 */

public class XMI2IR
  extends org.objectweb.openccm.command.lib.CompilerBase
    implements org.objectweb.openccm.uml.api.XMI2IR
{
    // ==================================================================
    //
    // Java IDL Script Commands.
    //
    // ==================================================================

    /** The Java IDL Script command for checking the UML model. */
    private final String checkCommand = "import CIDLChecker; checker = CIDLChecker.CIDLChecker(); checker.checkModel();";

    /** The Java IDL Script command for creating the UML model into the OpenCCM Interface Repository. */
    private final String createCommand = "import CIDLFeeder; feeder = CIDLFeeder.CIDLFeeder(); feeder.createModel();";

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
 
    /** The Java IDL Script Interpreter. */
    fr.lifl.cim.jidlscript.JISInterpreter jis;

    /** The UML XMI file importer. */
    org.omg.uml.xmi.UMLXMIImport xmiImport;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    XMI2IR()
    {
        // Calls the CompilerBase constructor.
        super(new DefaultCommandLine("xmi2ir",
                                     "xmi_uml_document",
                                     "Feed the OpenCCM Interface Repository with an XMI 1.1 UML 1.4 document",
                                     true),
              false);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Initialization of the Java IDL Script Interpreter.
     *
     * @return TRUE if the Java IDL Script is initialized.
     */
    private boolean
    initJIS()
    {
        org.omg.CORBA.ORB orb = org.objectweb.openccm.corba.TheORB.getORB();
        try {
            // Creation of the interpreter.
            jis = new fr.lifl.cim.jidlscript.JISInterpreter(orb);
            String csDirectory = System.getProperty("transformation.cs.dir");
            java.net.URL urlDirectory = Thread.currentThread().getContextClassLoader().getResource(csDirectory);
            fr.lifl.cim.jidlscript.lang.JISModuleLoader.loader.addURL(urlDirectory.toString());
            return true;
        } catch (fr.lifl.cim.jidlscript.lang.JISError jisError) {
            getConsole().error("Error in Java IDL Script Interpreter initialization: " + jisError.getMessage());
            return false;
        }
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.Application
    //
    // ==================================================================

    /**
     * Runs the application.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public int
    run(java.lang.String[] args)
    {
        // Inits the OpenCCM's Abstract Syntax Tree,
        // the Java IDL Script Interpreter and the UML Repository.
        if(!initAST() || !initJIS() || !initUMLRepository())
            return -1;

        try {
            parseXMIFile(args[0]);
        } catch (java.lang.Exception exception) {
            getConsole().error("[ERROR] An error occurs during the reading of the UML model (" + exception.getMessage() + ").");
            return -1;
        }

        try {

            org.omg.CORBA.ComponentIR.Repository ir = org.objectweb.openccm.corba.TheInterfaceRepository.getRepository();

            // Checks if the model is correct.
            getConsole().message("Checking if the UML model is correct...");
            jis.eval(checkCommand);
            getConsole().message("The UML model is correct.");

            // Creates the corresponding elements into the OpenCCM Interface Repository.
            getConsole().message("Feeding the UML model into the OpenCCM Interface Repository...");
            jis.eval(createCommand);
            getConsole().message("The UML model is fed into the OpenCCM Interface Repository.");
        } catch (fr.lifl.cim.jidlscript.lang.JISError e) {
            if (((fr.lifl.cim.jidlscript.lang.JISException)e.exception).getType().Name().equals("Throw")) {
                String message = ((fr.lifl.cim.jidlscript.lang.JISThrow)e.exception).eval().toString();
                if (message.indexOf("java.lang.NullPointerException") != -1) {
                    getConsole().error("[ERROR] There is a blank field in the UML model (e.g. name, visibility, multiplicity, etc.).");
                } else {
                    // ConstraintException or NotCreatedException
                    java.util.StringTokenizer token = new java.util.StringTokenizer(message, "\n");
                    while (token.hasMoreTokens()) {
                        getConsole().error("[ERROR] " + token.nextToken());
                    }
                }
            } else {
                getConsole().error("[ERROR] An error occurs during the importation of the UML model. Check if your XMI file is correct (" + e.exception + ").");
            }
            return -1;
        }

        // All is OK.
        return 0;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.CommandOnIR3
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.CommandOnAST
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.Compiler
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.openccm.api.XMI2IR
    //
    // ==================================================================

    /**
     * Inits the UML repository.
     *
     * @return TRUE if the UML repository is initialized.
     */
    public boolean
    initUMLRepository()
    {
        try {
            org.omg.uml.UMLPackage pck = null;
            String metaModelFile = System.getProperty("uml.metamodel.file");
            pck = org.objectweb.modfact.jmi.repository.uml.UMLPackageImpl.create(metaModelFile);
            xmiImport = new org.omg.uml.xmi.UMLXMIImport();
            xmiImport.initiate(pck);
            return true;
        } catch (java.lang.Exception exception) {
            getConsole().error("[ERROR] An error occurs during the initialization of the UML repository. Check your environment.");
            exception.printStackTrace();
            return false;
        }
    }

    /**
     * Parses the UML XMI file for reading the UML Package.
     *
     * @param filename The filename of the UML XMI file.
     *
     * @throw org.xml.sax.SAXException If any parse errors occur.
     * @throw javax.xml.parsers.ParserConfigurationException If a DocumentBuilder cannot be
     *        created which satisfies the configuration requested.
     * @throw java.io.IOException If any IO errors occur.
     */
    public void
    parseXMIFile(String filename)
    throws
        org.xml.sax.SAXException,
        javax.xml.parsers.ParserConfigurationException,
        java.io.IOException
    {
        getConsole().message("Reading from " + filename + "...");
        Object o = xmiImport.parse(filename);
        org.omg.uml.UMLPackage uml_package = (org.omg.uml.UMLPackage) o;
        TheUMLPackage.setUMLPackage(uml_package);
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        XMI2IR xmi2ir = new XMI2IR();
        xmi2ir.runMain(args);
    }

}
