/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

--------------------------------------------------------------------
$Id: CommandFactory.java,v 1.1 2004/02/05 20:29:57 rouvoy Exp $
====================================================================*/

package org.objectweb.util.launcher ;


import java.util.Properties ;


/**
 * Implementation of a particular Java Command which takes into account.
 * <ul>
 *   <li>an Identifier for the command</li>
 *   <li>a bootstrap class name for executing the command</li>
 *   <li>a specific classloader where class are stored</li>
 *   <li>a list of java properties to associate to the command</li>
 *   <li>a list of arguments to give to the bootstrap class</li>
 *   <li>a particular mode of execution</li>
 * </ul>
 *
 * <p>
 * This command represent most of the command which could be executed 
 * in the context of Java developements.
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.2
 */
public class CommandFactory
{
    /** Singleton for the CommandFactory */
    protected static CommandFactory factory_ = null ;
    
    /** List of default Properties */
    protected Properties default_properties_ ;
    /** List of default Arguments */
    protected StringList default_arguments_ ;
    /** Description of the default classpath */
    protected StringList default_classpath_ ;
    

    /**
     * Default Constructor
     *
     */
    public CommandFactory() {
        this.default_properties_ = new Properties();
        this.default_arguments_ = new StringList();
        this.default_classpath_ = new StringList();
    }

    /**
     * Retrieves default properties for the factory.
     * @return previously defined properties.
     */
    public Properties getDefaultProperties() { 
        return this.default_properties_;
    }

    /**
     * Retrieves default arguments for the factory.
     * @return previously defined arguments.
     */
    public StringList getDefaultArguments() { 
        return this.default_arguments_;
    }

    /**
     * Retrieves the default classpath.
     * @return previously defined classpath.
     */
    public StringList getDefaultClasspath() { 
        return this.default_classpath_;
    }

    /**
     * Creates a new instance of pre-configured command.
     * @return the instance of a pre-configured command.
     */
    public CommandJava create() {
        CommandJava cmd = new CommandJava(new XBootClassLoader());
        cmd.addProperties(getDefaultProperties());
        cmd.addArguments(getDefaultArguments());
        try {
            cmd.getLoader().addURL(getDefaultClasspath().toStringArray());
        } catch (java.net.MalformedURLException ex) {
            throw new LauncherException(ex);
        }        
        return cmd;
    }

    /**
     * Singleton access method to the CommandFactory.
     * @return the default instance of CommandFactory.
     */
    public static CommandFactory instance() {
        if (factory_ == null) factory_ = new CommandFactory();
        return factory_ ;
    }
}
